#ifndef __DRV_GMAC_H_
#define __DRV_GMAC_H_


/// Number of buffer for RX, be carreful: MUST be 2^n
#define GRX_BUFFERS  16
/// Number of buffer for TX, be carreful: MUST be 2^n
#define GTX_BUFFERS   8

/// Buffer Size
#define MACB_DMADESC_SIZE           8
#define GMAC_RX_UNITSIZE            128     /// Fixed size for RX buffer
#define GMAC_TX_UNITSIZE            0x600   /// Size for ETH frame buffer length

#define GMAC_RX_UNITSIZE_SHIFT      7

//#define GMAC_TX_PACKAGE_SIZE        1518    /// Size for ETH frame length
 #define RX_PACKET_SIZE          0x600       /* 1518 rounded up */

#define DMA_GMACB_SIZE          ((MACB_DMADESC_SIZE+GMAC_RX_UNITSIZE)*GRX_BUFFERS + \
                                 (MACB_DMADESC_SIZE+GMAC_TX_UNITSIZE)*GTX_BUFFERS )

/** Receive buffer descriptor struct */
typedef struct _GMACRXDESC_
{
    union
    {
        UINT32  Val;
        struct
        {
            UINT32  bOwnership:1,  /**< User clear, GMAC set this to one once
                                         it has successfully written a frame to
                                         memory */
                    bWrap:1,       /**< Marks last descriptor in receive buffer */
                    addrDW:30;     /**< Address in number of DW */
        } bm;
    } Addr;                    /**< Address, Wrap & Ownership */
    union
    {
        UINT32  Val;
        struct
        {
            UINT32   len:12,                /** Length of frame including FCS */
                     offset:2,              /** Receive buffer offset,
                                                bits 13:12 of frame length for jumbo
                                                frame */
                     bSof:1,                /** Start of frame */
                     bEof:1,                /** End of frame */
                     bCFI:1,                /** Concatenation Format Indicator */
                     vlanPriority:3,        /** VLAN priority (if VLAN detected) */
                     bPriorityDetected:1,   /** Priority tag detected */
                     bVlanDetected:1,       /**< VLAN tag detected */
                     bTypeIDMatch:1,        /**< Type ID match */
                     bAddr4Match:1,         /**< Address register 4 match */
                     bAddr3Match:1,         /**< Address register 3 match */
                     bAddr2Match:1,         /**< Address register 2 match */
                     bAddr1Match:1,         /**< Address register 1 match */
                     reserved:1,
                     bExtAddrMatch:1,       /**< External address match */
                     bUniHashMatch:1,       /**< Unicast hash match */
                     bMultiHashMatch:1,     /**< Multicast hash match */
                     bBroadcastDetected:1;  /**< Global all ones broadcast
                                                 address detected */
        } bm;
    } Status;
} GMACRXDESC;

/** Transmit buffer descriptor struct */
typedef struct _GMACTXDESC_
{
    UINT32          Addr;
    union
    {
        UINT32  Val;
        struct
        {
            UINT32   len:11,        /**< Length of buffer */
                     reserved:4,
                     bLastBuffer:1, /**< Last buffer (in the current frame) */
                     bNoCRC:1,      /**< No CRC */
                     reserved1:10,
                     bExhausted:1,  /**< Buffer exhausted in mid frame */
                     bUnderrun:1,   /**< Transmit underrun */
                     bError:1,      /**< Retry limit exceeded, error detected */
                     bWrap:1,       /**< Marks last descriptor in TD list */
                     bUsed:1;       /**< User clear, GMAC sets this once a frame
                                         has been successfully transmitted */
        } bm;
    } Status;
} GMACTXDESC;

/** Describes the statistics of the GMAC. */
typedef struct _GmacStats
{

    // TX errors
    UINT32 tx_packets;    /** Total Number of packets sent */
    UINT32 tx_comp;       /** Packet complete*/
    UINT32 tx_errors;     /** TX errors ( Retry Limit Exceed )*/
    UINT32 collisions;    /** Collision*/
    UINT32 tx_exausts;    /** Buffer exhausted*/
    UINT32 tx_underruns;  /** Under Run, not able to read from memory*/
    UINT32 tx_hresp;      /** HRESP Not OK*/
    UINT32 tx_latecollisions;  /** Late Collision Occurred*/
    // RX errors
    UINT32 rx_packets;    /** Total Number of packets RX*/
    UINT32 rx_comp;       /** Total Number of packets RX*/
    UINT32 rx_eof;        /** No EOF error*/
    UINT32 rx_ovrs;       /** Over Run, not able to store to memory*/
    UINT32 rx_bnas;       /** Buffer is not available*/

} GmacStats;



// #define INCLUDE_MAC_REV3
// 
// #define INCLUDE_OPTIMIZE_FOR_ZERO_COPY
// 
// #define NO_MDINT		/* if NO_MDINT, always fix MAC SPEED/DUPLEX to 100/Full */
// 
// #define MAC_RX_BADDR_REG_FIFO_DEPTH	15
// #define MAC_TX_BADDR_REG_FIFO_DEPTH	15
// 
// 
// /* Board Specific Definition */
// 
// 
// /* defines */
// #define HBP_ETH_DEV_NAME "eth"
// #define HBP_ETH_DEV_NAME_LEN 4
// 
// 
// /*
//  * the following may be redefined in the bsp to account for the actual
//  * number of Ethernet PHY in the system.
//  */
// 
// #ifndef MAX_UNIT_NUM
// #define MAX_UNIT_NUM  2
// #endif
// 
// 
// /* defines */
// #define ENET_ADDR_SIZE      0x6	/* size of Ethernet src/dest addresses */
// #define TX_BD_MIN           2	/* minimum number of Tx buffer descriptors */
// #define TX_BD_MAX           128	/* maximum number of Tx buffer descriptors */
// #define RX_BD_MIN           2	/* minimum number of Rx buffer descriptors */
// #define TX_BD_DEFAULT       0x40	/* default number of Tx buffer descriptors */
// #define RX_BD_DEFAULT       0x40	/* default number of Rx buffer descriptors */
 #define FRAME_MAX           1518	/* maximum frame size */
// #define FRAME_MAX_AL        2000    /*1524*/ /* 1520 */	/* maximum frame size, 4 byte alligned + MAC use 4 byte after CRC */
 #define FRAME_MIN           64	/* minimum frame size */
// #define LENGTH_MIN_FBUF     9	/* min. size of the first buffer in a frame */
// #define SPEED               10000000	/* ethernet speed */
// #define L_POOL              0x80	/* number of Rx loaner buffers in pool */
// 
// /* nial buffer size - needed in nial_BufferConfigure */
// #define	NIAL_BUF_SMALL_SIZE 512
// #define	NIAL_BUF_SMALL_NUM  10
// #define	NIAL_BUF_MID_SIZE   1024
// #define	NIAL_BUF_MID_NUM    10
// #define	NIAL_BUF_LARGE_SIZE 2256
// #define	NIAL_BUF_LARGE_NUM  400
// 
// 
// typedef struct free_args 
// {
//     void *arg1;
//     void *arg2;
// } FREE_ARGS;
// 
// 
// #define ETH_HEADER_SIZE         14      /* ethernet header size */
// #define BUFFER_HEADER_SIZE      8       /* next, status */
// #define MAX_PAYLOAD_SIZE        1504	/* maximum size of data to receive + CRC */
// #define BUFFER_RSVD2_SIZE       518
// 
// #define PAYLOAD_ALIGN_OFFSET    (0x2)   /* HBP MAC require 2 byte reserved offset field before starting of ethernet header */
//                                         /* in order to make payload 4-byte aligned */
// 
// /* reserved area offset for RFC1483 */
// #define RX_RESERVED_OFFSET      8
// 
// 
// 
// #define _B_FREE_            0
// #define _B_FILLED_          1
// 
// #define HBP_INT_LVL_ENET0   11	/* MAC 0 */
// #define HBP_INT_LVL_ENET1   14	/* MAC 1 */
// 
// #define MAC0_BASE_ADDR      0x99200000	/* 0xA0000000 + 0x19300000 */
// #define MAC1_BASE_ADDR      0x99201000
// 
// #define MAC_REG( pEthNetInfo, nReg )     (*(VOLATILE PUINT32)(((VOLATILE PUINT32)pEthNetInfo->RegBase) + nReg))
// 
// //#define MAC_REG_READ(pEthNetInfo, nReg, Result)	(((VOLATILE UINT32)Result) = (VOLATILE UINT32)MAC_REG( pEthNetInfo, nReg ))
// #define MAC_REG_READ(pEthNetInfo, nReg, Result)	((Result) = (VOLATILE UINT32)MAC_REG( pEthNetInfo, nReg ))
// 
// #define MAC_REG_WRITE(pEthNetInfo, nReg, Data)  (MAC_REG( pEthNetInfo, nReg ) = ((VOLATILE UINT32)Data))
// 
// 
// //#define MAC_REG_WRITE(nReg, Data)   (((*(VOLATILE PUINT32)pEthNetInfo->RegBase) + nReg) = (Data))
// 
// /* ethernet registers */
// #define MAC_MODE        0x00	/*      32      RW      MAC mode register                                       */
// #define INT_SRC         0x01	/*      32      RWC     Interrupt source register               */
// #define HBP_MAC_INT_ENABLE_REG  0x02	/*      32      RW      Interrupt mask register                         */
// #define IF_GAP          0x03	/*      32      RW      Inter packet gap register                       */
// #define COLL_CFG        0x04	/*      32      RW      Collision control register                              */
// #define TX_BADDR        0x05	/*      32      RW      Transmit buffer base address            */
// #define TX_LENGTH       0x06	/*      32      RW      Transmit buffer length                          */
// #define RX_BADDR        0x07	/*      32      RW      Receive buffer base address                     */
// #define RX_BSTAT        0x08	/*      32      RW      Receive buffer status                           */
// #define RX_BUFLVL       0x09	/*      32      RW      Address/Status Buffer Level                     */
// #define RX_DONE_BADDR   0x0A	/*      32      R       Used Rx Buffer Address                          */
// #define CTRL_MODE       0x0B	/*      32      RW      Control mode register                           */
// #define MII_MODE        0x0C	/*      32      RW      MII mode register                                       */
// #define MII_CMD         0x0D	/*      32      RW      MII command register                            */
// #define MII_TXDATA      0x0E	/*      32      RW      MII transmit data                                       */
// #define MII_RXDATA      0x0F	/*      32      RW      MII receive data                                        */
// /*#define RESERVED2     0x10	*//*      32      RW      MII status register                                     */
// #define LENGTH          0x11	/*      32      RW      Max, Min, burst length register         */
// #define MCAST_ADDR_0    0x12	/*      32      RW      Multicast Address (Most significant)    */
// #define MCAST_ADDR_1    0x13	/*      32      RW      Multicast Address (Least significant)   */
// #define MAC_ADDR_00     0x14	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_01     0x15	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_10     0x16	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_11     0x17	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_20     0x18	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_21     0x19	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_30     0x1A	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_31     0x1B	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_40     0x1C	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_41     0x1D	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_50     0x1E	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_51     0x1F	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_60     0x20	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_61     0x21	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define MAC_ADDR_70     0x22	/*      32      RW      MAC address (upper 2 bytes)             */
// #define MAC_ADDR_71     0x23	/*      32      RW      MAC address (lower 4 bytes)                     */
// #define P_FRM_ADDR_0    0x24	/*      32      RW      Pause frame address (upper 2 bytes)     */
// #define P_FRM_ADDR_1    0x25	/*      32      RW      Pause frame address (lower 4 bytes)     */
// #define P_FRM_ID        0x26	/*      32      RW      Pause frame Type / Op. code             */
// #define P_FRM_VALUE     0x27	/*      32      RW      Pause frame delay value                 */
// 
// #ifdef INCLUDE_MAC_REV3
// #define TX_BADDR_H      0x28	/*      RW High Priority Tx Base Address                        */
// #define TX_LENGTH_H     0x29	/*      RW High Priority Tx Frame Length                                */
// #define TX_BUFLVL       0x2A	/*      RW Buffer level of Tx Base Address, Length, Address Back */
// #define TX_DONE_BADDR   0x2B
// #define TX_BUFLVL_H     0x2C
// #define TX_DONE_BADDR_H 0x2D
// 
// #endif
// 
// #ifdef INCLUDE_MAC_REV4
// #define RX_DONE_TIMER   0x2E
// #define RX_TRIGLVL      0x2F
// #define TX_DONE_TIMER   0x30
// #define TX_TRIGLVL_H    0x31
// #define TX_TRIGLVL      0x32
// #endif
// 
// 
// /* declarations of interrupt bit */
// #define MAC_INT_MII_PHY                         0x80000000
// #define MAC_INT_MII_SCAN						0x40000000
// #define MAC_INT_PHY_TX_ERROR					0x20000000
// #define MAC_INT_PHY_RX_ERROR					0x10000000
// #define MAC_INT_COLLISION_DETECTED				0x02000000
// #define MAC_INT_CARRIER_SENSE_DETECTED			0x01000000
// #define MAC_INT_LATE_COLLISION_DETECTED			0x00200000
// #define MAC_INT_TX_DONE							0x00100000
// #define MAC_INT_TX_SYS_FIFO_OVERFLOW			0x00080000
// #define MAC_INT_TX_SYS_FIFO_UNDERFLOW			0x00040000
// 
// /*#define MAC_INT_TX_MAC_FIFO_OVERFLOW		0x00020000
// #define MAC_INT_TX_MAC_FIFO_UNDERFLOW		0x00010000
// */
// #define MAC_INT_RX_DONE_BADDR_BUFF_OVERFLOW     0x00020000
// #define MAC_INT_RX_DONE_BADDR_BUFF_UNDERFLOW	0x00010000
// #define MAC_INT_TX_LENGTH_BUFF_OVERFLOW			0x00008000
// #define MAC_INT_TX_LENGTH_BUFF_UNDERFLOW		0x00004000
// #define MAC_INT_TX_BADDR_BUFF_OVERFLOW			0x00002000
// #define MAC_INT_TX_BADDR_BUFF_UNDERFLOW		    0x00001000
// 
// #define MAC_INT_RX_STATUS_BUFF_OVERFLOW			0x00000800
// #define MAC_INT_RX_STATUS_BUFF_UNDERFLOW		0x00000400
// #define MAC_INT_RX_BADDR_BUFF_OVERFLOW			0x00000200
// #define MAC_INT_RX_BADDR_BUFF_UNDERFLOW		    0x00000100
// #define MAC_INT_RX_FRAME_CUT					0x00000040
// #define MAC_INT_RX_FRAME_DROPPED				0x00000020
// #define MAC_INT_RX_DONE							0x00000010
// #define MAC_INT_RX_SYS_FIFO_OVERFLOW			0x00000008
// #define MAC_INT_RX_SYS_FIFO_UNDERFLOW			0x00000004
// #define MAC_INT_RX_MAC_FIFO_OVERFLOW			0x00000002
// #define MAC_INT_RX_MAC_FIFO_UNDERFLOW			0x00000001
// 
// #define MAC_INT_FATAL_ERROR (MAC_INT_TX_SYS_FIFO_OVERFLOW | \
// MAC_INT_TX_SYS_FIFO_UNDERFLOW | \
// MAC_INT_RX_DONE_BADDR_BUFF_OVERFLOW | \
// MAC_INT_RX_DONE_BADDR_BUFF_UNDERFLOW | \
// MAC_INT_RX_SYS_FIFO_OVERFLOW | \
// MAC_INT_RX_SYS_FIFO_UNDERFLOW | \
// MAC_INT_RX_MAC_FIFO_OVERFLOW | \
// MAC_INT_RX_MAC_FIFO_UNDERFLOW | \
// MAC_INT_RX_STATUS_BUFF_OVERFLOW | \
// MAC_INT_RX_STATUS_BUFF_UNDERFLOW | \
// MAC_INT_RX_BADDR_BUFF_OVERFLOW | \
// MAC_INT_RX_BADDR_BUFF_UNDERFLOW | \
// MAC_INT_TX_LENGTH_BUFF_OVERFLOW | \
// MAC_INT_TX_LENGTH_BUFF_UNDERFLOW | \
// MAC_INT_TX_BADDR_BUFF_OVERFLOW | \
// MAC_INT_TX_BADDR_BUFF_UNDERFLOW )
// 
// 
// #ifndef INCLUDE_OPTIMIZE_FOR_ZERO_COPY
// #define MAC_INT_ENABLED_BITS				    0xB23FFF7F	/* Disable MII_SCAN & CARRIER_SENSE */
// #else
// 
// /*#define MAC_INT_ENABLED_BITS 0x80100070
// */
// 
// #define MAC_INT_ENABLED_BITS	( MAC_INT_MII_PHY  | \
// MAC_INT_TX_DONE | \
// MAC_INT_RX_FRAME_CUT | \
// MAC_INT_RX_FRAME_DROPPED | \
// MAC_INT_RX_DONE | \
// MAC_INT_FATAL_ERROR )
// 
// #endif
// 
// /* Definitions of MAC_MODE bits */
// #define MAC_MODE_RX_UNICAST				            0x80000000
// #define MAC_MODE_RX_MULTICAST				        0x40000000
// #define MAC_MODE_RX_BROADCAST				        0x20000000
// #define MAC_MODE_RX_ALL					            0x10000000
// 
// #define MAC_MODE_TX_ERROR_POLARITY_ACTIVE_HIGH		0x00040000
// #define MAC_MODE_TX_DATA_POLARITY_ACTIVE_HIGH		0x00020000
// 
// #define MAC_MODE_TX_ENABLE				            0x00010000
// 
// #define MAC_MODE_RX_PREAMBLE_ENABLE			        0x00001000
// 
// #define MAC_MODE_RX_VARIABLE_BUFF_SIZE			    0x00000800
// #define MAC_MODE_RX_ERROR_POLARITY_ACTIVE_HIGH		0x00000400
// #define MAC_MODE_RX_DATA_POLARITY_ACTIVE_HIGH		0x00000200
// 
// #define MAC_MODE_RX_ENABLE				            0x00000100
// #define MAC_MODE_100MBPS				            0x00000080
// #define MAC_MODE_RMII                  			    0x00000040
// #define MAC_MODE_COLLISION_POLARITY_ACTIVE_HIGH 	0x00000020
// #define MAC_MODE_CARRIER_SENSE_POLARITY_ACTIVE_HIGH	0x00000010
// #define MAC_MODE_LOOPBACK				            0x00000008
// #define MAC_MODE_FULLDUPLEX				            0x00000004
// #define MAC_MODE_MII					            0x00000002
// #define MAC_MODE_RESET					            0x00000001
// 
// 
// 
// 
// 
// /* Definition of PHY Configuration */
// #define PHY_LXT970A             1
// #define PHY_LXT972A             2
// #define PHY_KS8737              3
// 
// #define PHY_AUTO_NEGO	        0x1000
// 
// #define PHY_FULL_DUPLEX		    0x100
// #define PHY_SPEED_100M		    0x2000
// #define PHY_MODE 		        PHY_AUTO_NEGO  /*(PHY_FULL_DUPLEX|PHY_SPEED_100M)*/	/* Default is 10M/Half */
//     /* PHY_MODE can be PHY_FULL_DUPLEX | PHY_SPEED_100M */
//     /*                                      or PHY_AUTO_NEGO */
// 
// #define PHY_AUTO_NEGO_TIMEOUT	300	/* unit of 10 ms */
//     /* 100 == 1 sec */
// #define PHY_RESET_TIMEOUT		10	/* 10 == 100 ms */
// 
// /* Definition of MAC Configuration */
// #define PHY_LOOPBACK	        0x4000
// #define MAC_LOOPBACK	        0x2
// #define ENABLE_LOOPBACK		    0
// 
// 
// #define MEM_BURST_LENGTH	    0x90
// 
// /* Definition of PHY REGISTERS */
// #define PHY_CONTROL							0
// #define PHY_STATUS							1
// #define PHY_MFG_ID1							2
// #define PHY_MFG_ID2							3
// #define PHY_AUTO_NEGO_AD					4
// #define PHY_AUTO_NEGO_PARTNER_ABILITY	    5
// #define PHY_AUTO_NEGO_EXPANSION			    6
// 
// #define PHY_LXT970A_INT_ENABLE			    17
// #define PHY_LXT970A_INT_STATUS			    18
// #define PHY_LXT970A_CONFIG					19
// #define PHY_LXT970A_CHIP_STATUS			    20
// 
// #define PHY_LXT972A_CONFIG 				    16
// #define PHY_LXT972A_STATUS_2				17
// #define PHY_LXT972A_INT_ENABLE			    18
// #define PHY_LXT972A_INT_STATUS			    19
// 
// #define PHY_KS8737_INT_ENABLE_STATUS		0x1B
// #define PHY_KS8737_100M_CONTROL			    0x1F
// 
// /* Bit Definition of PHY INT ENABLE Register */
// #define PHY_LXT972A_INT_AUTO_NEGO_COMPLETE	0x80
// #define PHY_LXT972A_INT_LINK_SPEED_CHANGED	0x40
// #define PHY_LXT972A_INT_DUPLEX_CHANGED		0x20
// #define PHY_LXT972A_INT_LINK_STATUS_CHANGED	0x10
// 
// #define PHY_KS8737_INT_LINK_UP 	            0x1
// #define PHY_KS8737_INT_LINK_DOWN            0x4
// 
// 
// #ifndef SYS_INT_DISABLE
// #define SYS_INT_DISABLE(ilvl) disable_irq(ilvl)
// #endif
// 
// #ifndef SYS_INT_ENABLE
// #define SYS_INT_ENABLE(ilvl) enable_irq(ilvl)
// #endif
// 
// #ifndef SYS_INT_ACK
// #define SYS_INT_ACK(ilvl)
// #endif
// 
// 
// /* Generic MII registers. */
// 
// #define MII_BMCR            0x00        /* Basic mode control register */
// #define MII_BMSR            0x01        /* Basic mode status register  */
// #define MII_PHYSID1         0x02        /* PHYS ID 1                   */
// #define MII_PHYSID2         0x03        /* PHYS ID 2                   */
// #define MII_ADVERTISE       0x04        /* Advertisement control reg   */
// #define MII_LPA             0x05        /* Link partner ability reg    */
// #define MII_EXPANSION       0x06        /* Expansion register          */
// #define MII_CTRL1000        0x09        /* 1000BASE-T control          */
// #define MII_STAT1000        0x0a        /* 1000BASE-T status           */
// #define MII_ESTATUS         0x0f        /* Extended Status */
// #define MII_DCOUNTER        0x12        /* Disconnect counter          */
// #define MII_FCSCOUNTER      0x13        /* False carrier counter       */
// #define MII_NWAYTEST        0x14        /* N-way auto-neg test reg     */
// #define MII_RERRCOUNTER     0x15        /* Receive error counter       */
// #define MII_SREVISION       0x16        /* Silicon revision            */
// #define MII_RESV1           0x17        /* Reserved...                 */
// #define MII_LBRERROR        0x18        /* Lpback, rx, bypass error    */
// #define MII_PHYADDR         0x19        /* PHY address                 */
// #define MII_RESV2           0x1a        /* Reserved...                 */
// #define MII_TPISTATUS       0x1b        /* TPI status for 10mbps       */
// #define MII_NCONFIG         0x1c        /* Network interface config    */
// 
// /* Basic mode control register. */
// #define BMCR_RESV               0x003f  /* Unused...                   */
// #define BMCR_SPEED1000		    0x0040  /* MSB of Speed (1000)         */
// #define BMCR_CTST               0x0080  /* Collision test              */
// #define BMCR_FULLDPLX           0x0100  /* Full duplex                 */
// #define BMCR_ANRESTART          0x0200  /* Auto negotiation restart    */
// #define BMCR_ISOLATE            0x0400  /* Disconnect DP83840 from MII */
// #define BMCR_PDOWN              0x0800  /* Powerdown the DP83840       */
// #define BMCR_ANENABLE           0x1000  /* Enable auto negotiation     */
// #define BMCR_SPEED100           0x2000  /* Select 100Mbps              */
// #define BMCR_LOOPBACK           0x4000  /* TXD loopback bits           */
// #define BMCR_RESET              0x8000  /* Reset the DP83840           */
// 
// /* Basic mode status register. */
// #define BMSR_ERCAP              0x0001  /* Ext-reg capability          */
// #define BMSR_JCD                0x0002  /* Jabber detected             */
// #define BMSR_LSTATUS            0x0004  /* Link status                 */
// #define BMSR_ANEGCAPABLE        0x0008  /* Able to do auto-negotiation */
// #define BMSR_RFAULT             0x0010  /* Remote fault detected       */
// #define BMSR_ANEGCOMPLETE       0x0020  /* Auto-negotiation complete   */
// #define BMSR_RESV               0x00c0  /* Unused...                   */
// #define BMSR_ESTATEN            0x0100  /* Extended Status in R15 */
// #define BMSR_100HALF2           0x0200  /* Can do 100BASE-T2 HDX */
// #define BMSR_100FULL2           0x0400  /* Can do 100BASE-T2 FDX */
// #define BMSR_10HALF             0x0800  /* Can do 10mbps, half-duplex  */
// #define BMSR_10FULL             0x1000  /* Can do 10mbps, full-duplex  */
// #define BMSR_100HALF            0x2000  /* Can do 100mbps, half-duplex */
// #define BMSR_100FULL            0x4000  /* Can do 100mbps, full-duplex */
// #define BMSR_100BASE4           0x8000  /* Can do 100mbps, 4k packets  */
// 
// /* Advertisement control register. */
// #define ADVERTISE_SLCT          0x001f  /* Selector bits               */
// #define ADVERTISE_CSMA          0x0001  /* Only selector supported     */
// #define ADVERTISE_10HALF        0x0020  /* Try for 10mbps half-duplex  */
// #define ADVERTISE_1000XFULL     0x0020  /* Try for 1000BASE-X full-duplex */
// #define ADVERTISE_10FULL        0x0040  /* Try for 10mbps full-duplex  */
// #define ADVERTISE_1000XHALF     0x0040  /* Try for 1000BASE-X half-duplex */
// #define ADVERTISE_100HALF       0x0080  /* Try for 100mbps half-duplex */
// #define ADVERTISE_1000XPAUSE    0x0080  /* Try for 1000BASE-X pause    */
// #define ADVERTISE_100FULL       0x0100  /* Try for 100mbps full-duplex */
// #define ADVERTISE_1000XPSE_ASYM 0x0100  /* Try for 1000BASE-X asym pause */
// #define ADVERTISE_100BASE4      0x0200  /* Try for 100mbps 4k packets  */
// #define ADVERTISE_PAUSE_CAP     0x0400  /* Try for pause               */
// #define ADVERTISE_PAUSE_ASYM    0x0800  /* Try for asymetric pause     */
// #define ADVERTISE_RESV          0x1000  /* Unused...                   */
// #define ADVERTISE_RFAULT        0x2000  /* Say we can detect faults    */
// #define ADVERTISE_LPACK         0x4000  /* Ack link partners response  */
// #define ADVERTISE_NPAGE         0x8000  /* Next page bit               */
// 
// #define ADVERTISE_FULL (ADVERTISE_100FULL | ADVERTISE_10FULL | \
// 			ADVERTISE_CSMA)
// #define ADVERTISE_ALL (ADVERTISE_10HALF | ADVERTISE_10FULL | \
//                        ADVERTISE_100HALF | ADVERTISE_100FULL)
// 
// /* Link partner ability register. */
// #define LPA_SLCT                0x001f  /* Same as advertise selector  */
// #define LPA_10HALF              0x0020  /* Can do 10mbps half-duplex   */
// #define LPA_1000XFULL           0x0020  /* Can do 1000BASE-X full-duplex */
// #define LPA_10FULL              0x0040  /* Can do 10mbps full-duplex   */
// #define LPA_1000XHALF           0x0040  /* Can do 1000BASE-X half-duplex */
// #define LPA_100HALF             0x0080  /* Can do 100mbps half-duplex  */
// #define LPA_1000XPAUSE          0x0080  /* Can do 1000BASE-X pause     */
// #define LPA_100FULL             0x0100  /* Can do 100mbps full-duplex  */
// #define LPA_1000XPAUSE_ASYM     0x0100  /* Can do 1000BASE-X pause asym*/
// #define LPA_100BASE4            0x0200  /* Can do 100mbps 4k packets   */
// #define LPA_PAUSE_CAP           0x0400  /* Can pause                   */
// #define LPA_PAUSE_ASYM          0x0800  /* Can pause asymetrically     */
// #define LPA_RESV                0x1000  /* Unused...                   */
// #define LPA_RFAULT              0x2000  /* Link partner faulted        */
// #define LPA_LPACK               0x4000  /* Link partner acked us       */
// #define LPA_NPAGE               0x8000  /* Next page bit               */
// 
// #define LPA_DUPLEX		(LPA_10FULL | LPA_100FULL)
// #define LPA_100			(LPA_100FULL | LPA_100HALF | LPA_100BASE4)
// 
// /* Expansion register for auto-negotiation. */
// #define EXPANSION_NWAY          0x0001  /* Can do N-way auto-nego      */
// #define EXPANSION_LCWP          0x0002  /* Got new RX page code word   */
// #define EXPANSION_ENABLENPAGE   0x0004  /* This enables npage words    */
// #define EXPANSION_NPCAPABLE     0x0008  /* Link partner supports npage */
// #define EXPANSION_MFAULTS       0x0010  /* Multiple faults detected    */
// #define EXPANSION_RESV          0xffe0  /* Unused...                   */
// 
// #define ESTATUS_1000_TFULL	0x2000	/* Can do 1000BT Full */
// #define ESTATUS_1000_THALF	0x1000	/* Can do 1000BT Half */
// 
// /* N-way test register. */
// #define NWAYTEST_RESV1          0x00ff  /* Unused...                   */
// #define NWAYTEST_LOOPBACK       0x0100  /* Enable loopback for N-way   */
// #define NWAYTEST_RESV2          0xfe00  /* Unused...                   */
// 
// /* 1000BASE-T Control register */
// #define ADVERTISE_1000FULL      0x0200  /* Advertise 1000BASE-T full duplex */
// #define ADVERTISE_1000HALF      0x0100  /* Advertise 1000BASE-T half duplex */
// 
// /* 1000BASE-T Status register */
// #define LPA_1000LOCALRXOK       0x2000  /* Link partner local receiver status */
// #define LPA_1000REMRXOK         0x1000  /* Link partner remote receiver status */
// #define LPA_1000FULL            0x0800  /* Link partner 1000BASE-T full duplex */
// #define LPA_1000HALF            0x0400  /* Link partner 1000BASE-T half duplex */
// 
// 
// 
// /* MACB register offsets */
// #define MACB_NCR				0x0000
// #define MACB_NCFGR				0x0004
// #define MACB_NSR				0x0008
// #define MACB_TSR				0x0014
// #define MACB_RBQP				0x0018
// #define MACB_TBQP				0x001c
// #define MACB_RSR				0x0020
// #define MACB_ISR				0x0024
// #define MACB_IER				0x0028
// #define MACB_IDR				0x002c
// #define MACB_IMR				0x0030
// #define MACB_MAN				0x0034
// #define MACB_PTR				0x0038
// #define MACB_PFR				0x003c
// #define MACB_FTO				0x0040
// #define MACB_SCF				0x0044
// #define MACB_MCF				0x0048
// #define MACB_FRO				0x004c
// #define MACB_FCSE				0x0050
// #define MACB_ALE				0x0054
// #define MACB_DTF				0x0058
// #define MACB_LCOL				0x005c
// #define MACB_EXCOL				0x0060
// #define MACB_TUND				0x0064
// #define MACB_CSE				0x0068
// #define MACB_RRE				0x006c
// #define MACB_ROVR				0x0070
// #define MACB_RSE				0x0074
// #define MACB_ELE				0x0078
// #define MACB_RJA				0x007c
// #define MACB_USF				0x0080
// #define MACB_STE				0x0084
// #define MACB_RLE				0x0088
// #define MACB_TPF				0x008c
// #define MACB_HRB				0x0090
// #define MACB_HRT				0x0094
// #define MACB_SA1B				0x0098
// #define MACB_SA1T				0x009c
// #define MACB_SA2B				0x00a0
// #define MACB_SA2T				0x00a4
// #define MACB_SA3B				0x00a8
// #define MACB_SA3T				0x00ac
// #define MACB_SA4B				0x00b0
// #define MACB_SA4T				0x00b4
// #define MACB_TID				0x00b8
// #define MACB_TPQ				0x00bc
// #define MACB_USRIO				0x00c0
// #define MACB_WOL				0x00c4
// 
// /* Bitfields in NCR */
// #define MACB_LB_OFFSET				0
// #define MACB_LB_SIZE				1
// #define MACB_LLB_OFFSET				1
// #define MACB_LLB_SIZE				1
// #define MACB_RE_OFFSET				2
// #define MACB_RE_SIZE				1
// #define MACB_TE_OFFSET				3
// #define MACB_TE_SIZE				1
// #define MACB_MPE_OFFSET				4
// #define MACB_MPE_SIZE				1
// #define MACB_CLRSTAT_OFFSET			5
// #define MACB_CLRSTAT_SIZE			1
// #define MACB_INCSTAT_OFFSET			6
// #define MACB_INCSTAT_SIZE			1
// #define MACB_WESTAT_OFFSET			7
// #define MACB_WESTAT_SIZE			1
// #define MACB_BP_OFFSET				8
// #define MACB_BP_SIZE				1
// #define MACB_TSTART_OFFSET			9
// #define MACB_TSTART_SIZE			1
// #define MACB_THALT_OFFSET			10
// #define MACB_THALT_SIZE				1
// #define MACB_NCR_TPF_OFFSET			11
// #define MACB_NCR_TPF_SIZE			1
// #define MACB_TZQ_OFFSET				12
// #define MACB_TZQ_SIZE				1
// 
// /* Bitfields in NCFGR */
// #define MACB_SPD_OFFSET				0
// #define MACB_SPD_SIZE				1
// #define MACB_FD_OFFSET				1
// #define MACB_FD_SIZE				1
// #define MACB_BIT_RATE_OFFSET			2
// #define MACB_BIT_RATE_SIZE			1
// #define MACB_JFRAME_OFFSET			3
// #define MACB_JFRAME_SIZE			1
// #define MACB_CAF_OFFSET				4
// #define MACB_CAF_SIZE				1
// #define MACB_NBC_OFFSET				5
// #define MACB_NBC_SIZE				1
// #define MACB_NCFGR_MTI_OFFSET			6
// #define MACB_NCFGR_MTI_SIZE			1
// #define MACB_UNI_OFFSET				7
// #define MACB_UNI_SIZE				1
// #define MACB_BIG_OFFSET				8
// #define MACB_BIG_SIZE				1
// #define MACB_EAE_OFFSET				9
// #define MACB_EAE_SIZE				1
// #define MACB_CLK_OFFSET				10
// #define MACB_CLK_SIZE				2
// #define MACB_RTY_OFFSET				12
// #define MACB_RTY_SIZE				1
// #define MACB_PAE_OFFSET				13
// #define MACB_PAE_SIZE				1
// #define MACB_RBOF_OFFSET			14
// #define MACB_RBOF_SIZE				2
// #define MACB_RLCE_OFFSET			16
// #define MACB_RLCE_SIZE				1
// #define MACB_DRFCS_OFFSET			17
// #define MACB_DRFCS_SIZE				1
// #define MACB_EFRHD_OFFSET			18
// #define MACB_EFRHD_SIZE				1
// #define MACB_IRXFCS_OFFSET			19
// #define MACB_IRXFCS_SIZE			1
// 
// /* Bitfields in NSR */
// #define MACB_NSR_LINK_OFFSET			0
// #define MACB_NSR_LINK_SIZE			1
// #define MACB_MDIO_OFFSET			1
// #define MACB_MDIO_SIZE				1
// #define MACB_IDLE_OFFSET			2
// #define MACB_IDLE_SIZE				1
// 
// /* Bitfields in TSR */
// #define MACB_UBR_OFFSET				0
// #define MACB_UBR_SIZE				1
// #define MACB_COL_OFFSET				1
// #define MACB_COL_SIZE				1
// #define MACB_TSR_RLE_OFFSET			2
// #define MACB_TSR_RLE_SIZE			1
// #define MACB_TGO_OFFSET				3
// #define MACB_TGO_SIZE				1
// #define MACB_BEX_OFFSET				4
// #define MACB_BEX_SIZE				1
// #define MACB_COMP_OFFSET			5
// #define MACB_COMP_SIZE				1
// #define MACB_UND_OFFSET				6
// #define MACB_UND_SIZE				1
// 
// /* Bitfields in RSR */
// #define MACB_BNA_OFFSET				0
// #define MACB_BNA_SIZE				1
// #define MACB_REC_OFFSET				1
// #define MACB_REC_SIZE				1
// #define MACB_OVR_OFFSET				2
// #define MACB_OVR_SIZE				1
// 
// /* Bitfields in ISR/IER/IDR/IMR */
// #define MACB_MFD_OFFSET				0
// #define MACB_MFD_SIZE				1
// #define MACB_RCOMP_OFFSET			1
// #define MACB_RCOMP_SIZE				1
// #define MACB_RXUBR_OFFSET			2
// #define MACB_RXUBR_SIZE				1
// #define MACB_TXUBR_OFFSET			3
// #define MACB_TXUBR_SIZE				1
// #define MACB_ISR_TUND_OFFSET			4
// #define MACB_ISR_TUND_SIZE			1
// #define MACB_ISR_RLE_OFFSET			5
// #define MACB_ISR_RLE_SIZE			1
// #define MACB_TXERR_OFFSET			6
// #define MACB_TXERR_SIZE				1
// #define MACB_TCOMP_OFFSET			7
// #define MACB_TCOMP_SIZE				1
// #define MACB_ISR_LINK_OFFSET			9
// #define MACB_ISR_LINK_SIZE			1
// #define MACB_ISR_ROVR_OFFSET			10
// #define MACB_ISR_ROVR_SIZE			1
// #define MACB_HRESP_OFFSET			11
// #define MACB_HRESP_SIZE				1
// #define MACB_PFR_OFFSET				12
// #define MACB_PFR_SIZE				1
// #define MACB_PTZ_OFFSET				13
// #define MACB_PTZ_SIZE				1
// 
// /* Bitfields in MAN */
// #define MACB_DATA_OFFSET			0
// #define MACB_DATA_SIZE				16
// #define MACB_CODE_OFFSET			16
// #define MACB_CODE_SIZE				2
// #define MACB_REGA_OFFSET			18
// #define MACB_REGA_SIZE				5
// #define MACB_PHYA_OFFSET			23
// #define MACB_PHYA_SIZE				5
// #define MACB_RW_OFFSET				28
// #define MACB_RW_SIZE				2
// #define MACB_SOF_OFFSET				30
// #define MACB_SOF_SIZE				2
// 
// /* Bitfields in USRIO (AVR32) */
// #define MACB_MII_OFFSET				0
// #define MACB_MII_SIZE				1
// #define MACB_EAM_OFFSET				1
// #define MACB_EAM_SIZE				1
// #define MACB_TX_PAUSE_OFFSET			2
// #define MACB_TX_PAUSE_SIZE			1
// #define MACB_TX_PAUSE_ZERO_OFFSET		3
// #define MACB_TX_PAUSE_ZERO_SIZE			1
// 
// /* Bitfields in USRIO (AT91) */
// #define MACB_RMII_OFFSET			0
// #define MACB_RMII_SIZE				1
// #define MACB_CLKEN_OFFSET			1
// #define MACB_CLKEN_SIZE				1
// 
// /* Bitfields in WOL */
// #define MACB_IP_OFFSET				0
// #define MACB_IP_SIZE				16
// #define MACB_MAG_OFFSET				16
// #define MACB_MAG_SIZE				1
// #define MACB_ARP_OFFSET				17
// #define MACB_ARP_SIZE				1
// #define MACB_SA1_OFFSET				18
// #define MACB_SA1_SIZE				1
// #define MACB_WOL_MTI_OFFSET			19
// #define MACB_WOL_MTI_SIZE			1
// 
// /* Constants for CLK */
// #define MACB_CLK_DIV8				0
// #define MACB_CLK_DIV16				1
// #define MACB_CLK_DIV32				2
// #define MACB_CLK_DIV64				3
// 
// /* Constants for MAN register */
// #define MACB_MAN_SOF				1
// #define MACB_MAN_WRITE				1
// #define MACB_MAN_READ				2
// #define MACB_MAN_CODE				2
// 
// /* Bit manipulation macros */
// #define MACB_BIT(name)					\
// 	(1 << MACB_##name##_OFFSET)
// #define MACB_BF(name,value)				\
// 	(((value) & ((1 << MACB_##name##_SIZE) - 1))	\
// 	 << MACB_##name##_OFFSET)
// #define MACB_BFEXT(name,value)\
// 	(((value) >> MACB_##name##_OFFSET)		\
// 	 & ((1 << MACB_##name##_SIZE) - 1))
// #define MACB_BFINS(name,value,old)			\
// 	(((old) & ~(((1 << MACB_##name##_SIZE) - 1)	\
// 		    << MACB_##name##_OFFSET))		\
// 	 | MACB_BF(name,value))
// 
// 
// 
// /* DMA descriptor bitfields */
// #define MACB_RX_USED_OFFSET			0
// #define MACB_RX_USED_SIZE			1
// #define MACB_RX_WRAP_OFFSET			1
// #define MACB_RX_WRAP_SIZE			1
// #define MACB_RX_WADDR_OFFSET			2
// #define MACB_RX_WADDR_SIZE			30
// 
// #define MACB_RX_FRMLEN_OFFSET			0
// #define MACB_RX_FRMLEN_SIZE			12
// #define MACB_RX_OFFSET_OFFSET			12
// #define MACB_RX_OFFSET_SIZE			2
// #define MACB_RX_SOF_OFFSET			14
// #define MACB_RX_SOF_SIZE			1
// #define MACB_RX_EOF_OFFSET			15
// #define MACB_RX_EOF_SIZE			1
// #define MACB_RX_CFI_OFFSET			16
// #define MACB_RX_CFI_SIZE			1
// #define MACB_RX_VLAN_PRI_OFFSET			17
// #define MACB_RX_VLAN_PRI_SIZE			3
// #define MACB_RX_PRI_TAG_OFFSET			20
// #define MACB_RX_PRI_TAG_SIZE			1
// #define MACB_RX_VLAN_TAG_OFFSET			21
// #define MACB_RX_VLAN_TAG_SIZE			1
// #define MACB_RX_TYPEID_MATCH_OFFSET		22
// #define MACB_RX_TYPEID_MATCH_SIZE		1
// #define MACB_RX_SA4_MATCH_OFFSET		23
// #define MACB_RX_SA4_MATCH_SIZE			1
// #define MACB_RX_SA3_MATCH_OFFSET		24
// #define MACB_RX_SA3_MATCH_SIZE			1
// #define MACB_RX_SA2_MATCH_OFFSET		25
// #define MACB_RX_SA2_MATCH_SIZE			1
// #define MACB_RX_SA1_MATCH_OFFSET		26
// #define MACB_RX_SA1_MATCH_SIZE			1
// #define MACB_RX_EXT_MATCH_OFFSET		28
// #define MACB_RX_EXT_MATCH_SIZE			1
// #define MACB_RX_UHASH_MATCH_OFFSET		29
// #define MACB_RX_UHASH_MATCH_SIZE		1
// #define MACB_RX_MHASH_MATCH_OFFSET		30
// #define MACB_RX_MHASH_MATCH_SIZE		1
// #define MACB_RX_BROADCAST_OFFSET		31
// #define MACB_RX_BROADCAST_SIZE			1
// 
// #define MACB_TX_FRMLEN_OFFSET			0
// #define MACB_TX_FRMLEN_SIZE			11
// #define MACB_TX_LAST_OFFSET			15
// #define MACB_TX_LAST_SIZE			1
// #define MACB_TX_NOCRC_OFFSET			16
// #define MACB_TX_NOCRC_SIZE			1
// #define MACB_TX_BUF_EXHAUSTED_OFFSET		27
// #define MACB_TX_BUF_EXHAUSTED_SIZE		1
// #define MACB_TX_UNDERRUN_OFFSET			28
// #define MACB_TX_UNDERRUN_SIZE			1
// #define MACB_TX_ERROR_OFFSET			29
// #define MACB_TX_ERROR_SIZE			1
// #define MACB_TX_WRAP_OFFSET			30
// #define MACB_TX_WRAP_SIZE			1
// #define MACB_TX_USED_OFFSET			31
// #define MACB_TX_USED_SIZE			1
// 
// 
// 
// 
// 
// 
// typedef unsigned int        DMA_ADDRESS;
// 
// 
// typedef struct _MACB_DMADESC_       MACB_DMADESC;
// typedef struct _MACB_DMADESC_       *PMACB_DMADESC;
// struct _MACB_DMADESC_
// {
//     UINT32  Addr;
//     UINT32  Ctrl;
// };
// 
// 
// //typedef struct _MACB_RINGINFO_       MACB_RINGINFO;
// //typedef struct _MACB_RINGINFO_       *PMACB_RINGINFO;
// //struct _MACB_RINGINFO_
// //{
// //    PBYTE               pBuffer;
// //    DMA_ADDRESS         Mapping;
// //	//struct sk_buff		*skb;
// //	//dma_addr_t		mapping;
// //};
// 
// 
// //#define RX_RING_SIZE		    512
// //#define RX_RING_BYTES		    (sizeof(MACB_DMADESC) * RX_RING_SIZE)
// //#define RX_BUFFER_SIZE		    128
// #define RX_PACKET_SIZE          0x600       /* 1518 rounded up */
// //
// ///* Make the IP header word-aligned (the ethernet header is 14 bytes) */
// //#define RX_OFFSET		        2
// //
// //#define TX_RING_SIZE		    128
// //#define TX_RING_BYTES		    ( * TX_RING_Ssizeof(MACB_DMADESC)IZE)
// //#define TX_BUFFER_SIZE          0x600       /* 1518 rounded up */
// ////#define DEF_TX_RING_PENDING	    (TX_RING_SIZE - 1)
// //
// 
// 
// /* Ethernet Buffer Descriptor structure */
// typedef struct _ENET_BUFDESC_       ENET_BUFDESC;
// typedef struct _ENET_BUFDESC_      *PENET_BUFDESC;
// struct _ENET_BUFDESC_
// {
//     VOLATILE UINT32 Status;
//     PENET_BUFDESC   pNext;
//     //PBYTE           pCluster;
// 
//     UINT32          FrameLength;
//     UINT32          FrameError;
//     BYTE            Buf[FRAME_MAX_AL];
// }__attribute__ ((packed));
// 
// //#define _ENET_BUFDESC_SIZE      sizeof(ENET_BUFDESC)
// 
// typedef struct _ENET_DEV_STAT_      ENET_DEV_STAT;
// typedef struct _ENET_DEV_STAT_      *PENET_DEV_STAT;
// struct _ENET_DEV_STAT_
// {
//     // Rx
//     UINT32  nRxPacket;
//     UINT32  nRxByte;
// 
//     // Tx
//     UINT32  nTxPacket;
//     UINT32  nTxByte;
// 
// };
// 
// ///* SCC_DEV */
// //typedef struct _SCC_DEV_            SCC_DEV;
// //typedef struct _SCC_DEV_           *PSCC_DEV;
// //struct _SCC_DEV_
// //{
// //    PENET_BUFDESC   pTxBufDesc;         /* transmit BD base address */
// //    PENET_BUFDESC   pRxBufDesc;         /* receive BD base address */
// //    UINT32          nTxBufDesc;         /* number of transmit buf descriptors */
// //    UINT32          nRxBufDesc;         /* number of receive buf descriptors */
// //    UINT32          TxBufSize;          /* transmit buffer size */
// //    UINT32          RxBufSize;          /* receive buffer size */
// //};
// 
// typedef struct _BUF_DESC_            BUF_DESC;
// typedef struct _BUF_DESC_           *PBUF_DESC;
// struct _BUF_DESC_
// {
//     PENET_BUFDESC   pBufDesc;           /* transmit BD base address */
//     UINT32          nBufDesc;           /* number of transmit buf descriptors */
//     UINT32          BufSize;            /* receive buffer size */
// };



/*
 * This structure is private to each device. It is used to pass
 * packets in and out, so there is place for a packet
 */
typedef struct _ENET_INFO_          ENET_INFO;
typedef struct _ENET_INFO_         *PENET_INFO;
struct _ENET_INFO_
{
    //SCC_DEV         EthDev;              /* ETHERNET SCC device */
//    BUF_DESC        RxBuf;
//    BUF_DESC        TxBuf;

    UINT32          iDev;
    UINT32          iNetIf;
    UINT32          Handle;


    // defintion for MAC
    UINT32          TxHead, TxTail;
    GMACTXDESC     *pTxTd;//[TX_RING_SIZE];
    PBYTE           pTxBuffer;//[TX_RING_SIZE][TX_BUFFER_SIZE];
    //MACB_RINGINFO   TxRingInfo[TX_RING_SIZE];

    UINT32          RxTail;
    GMACRXDESC     *pRxTd;//[RX_RING_SIZE];
    PBYTE           pRxBuffer;//[RX_RING_SIZE][RX_BUFFER_SIZE];
    BYTE            RxPacket[RX_PACKET_SIZE];

//    DMA_ADDRESS          TxRingDMA;
//    DMA_ADDRESS          RxRingDMA;
//    DMA_ADDRESS          RxBufferDMA;


    // status
//    ENET_DEV_STAT       Stat;

	//unsigned int		rx_tail;
	//struct dma_desc		*rx_ring;
	//void			*rx_buffers;
    //
	//unsigned int		tx_head, tx_tail;
	//struct dma_desc		*tx_ring;
	//struct ring_info	*tx_skb;



    volatile INT32  RxStatus;      /* flag, indicates netTask active */
    volatile INT32  TxStatus;      /* flag, indicates hardware is sending now */

    UINT32          nTxFrame;
    UINT32          nTxDone;
    UINT32          nTxFree;

//    PENET_BUFDESC   pRxNextDevice;  /* accessed next by rcv device */
//    PENET_BUFDESC   pRxNextDriver;  /* accessed next by rcv handler */
//    PENET_BUFDESC   pRxNextDone;
//    PENET_BUFDESC   pTxNextDevice;  /* accessed next by snd device */
//    PENET_BUFDESC   pTxNextDriver;  /* accessed next by snd function */
//    PENET_BUFDESC   pTxNextDone;

    /* definition for PHY */
    UINT32          PhyType;
    UINT32          PhyAddr;
    UINT32          PhyId;
    UINT32          PhyIntfMode;

    UINT32          RegBase;
    UINT32          nINTPhys;
    UINT32          nINTVirt;

    GmacStats *pGmacStats;

};


#endif  //__DRV_GMAC_H_
