//    DM9161A

#include <types.h>
#include <Time.h>
#include <panic.h>
#include "phy.h"

#define DM9161A_CONTROL_REG                 0
#define DM9161A_CONTROL_REG_RESET                       (1<<15)
#define DM9161A_CONTROL_REG_AUTONEGOCIATION_ENABLE      (1<<12)
#define DM9161A_CONTROL_REG_RESTART_AUTONEGOCIATION     (1<<9)
#define DM9161A_CONTROL_REG_FULL_DUPLEX                 (1<<8)
#define DM9161A_CONTROL_REG_SPEED_SELECT                (1<<13)

#define DM9161A_STATUS_REG                  1
#define DM9161A_STATUS_REG_LINK                         (1<<2)
#define DM9161A_STATUS_REG_AUTONEGOCIATION_COMPLETE     (1<<5)

#define DM9161A_ID1_REG                     2
#define DM9161A_ID2_REG                     3
#define MII_DM9161A_ID      0x0181b880


#define DM9161A_LINK_PARTNER_REG            5
#define DM9161A_LINK_PARTNER_100FULL                    (1<<8)
#define DM9161A_LINK_PARTNER_100HALF                    (1<<7)
#define DM9161A_LINK_PARTNER_10FULL                     (1<<6)
#define DM9161A_LINK_PARTNER_10HALF                     (1<<5)

#define DM9161A_DAVICOM_SPEC_CONF_REG       16
#define DM9161A_DAVICOM_SPEC_CONF_RMII_ENABLED          (1<<8)



STATIC BOOL 
PHY_Reset(MAC_PHY_POP pMacPhyOp, BYTE PhyAddr)
{
    DEBUGMSG(1,"RESET PHY!!!\r\n");
    pMacPhyOp->WritePhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_CONTROL_REG, DM9161A_CONTROL_REG_RESET);
    
    return TRUE;
}

STATIC BOOL 
PHY_StartAutoNegociation(MAC_PHY_POP pMacPhyOp, BYTE PhyAddr)
{
    pMacPhyOp->WritePhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_CONTROL_REG, DM9161A_CONTROL_REG_AUTONEGOCIATION_ENABLE | DM9161A_CONTROL_REG_RESTART_AUTONEGOCIATION);    
    return TRUE;
}



STATIC BOOL 
PHY_WaitForLink(MAC_PHY_POP pMacPhyOp, BYTE PhyAddr, UINT32 TimeOut)
{
    UINT32 Value;

    TimeOut = TimeOut / 100;

    do
    {
        Sleep(100);
        
        /* Link status is latched, so read twice to get current value */
        pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_STATUS_REG, &Value);
        pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_STATUS_REG, &Value);
//        DEBUGMSG(1,"Status = %d\r\n",Value));
        if( Value & DM9161A_STATUS_REG_LINK )
        {
            break;
        }
        
    }while( TimeOut-- > 0 );

    if( (Value & DM9161A_STATUS_REG_LINK) == 0 )
    {
        return FALSE;
    }
    
    return TRUE;
}

STATIC BOOL 
PHY_WaitForAutonegociationComplete( MAC_PHY_POP pMacPhyOp, BYTE PhyAddr, UINT32 TimeOut )
{
    UINT32 Value;
    TimeOut = TimeOut/100;

    do
    {
        Sleep(100);
        
        /* Link status is latched, so read twice to get current value */
        pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_STATUS_REG, &Value);
        pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_STATUS_REG, &Value);
        if( Value & DM9161A_STATUS_REG_AUTONEGOCIATION_COMPLETE )
        {
            break;
        }
    }while( TimeOut-- > 0 );

    if( (Value & DM9161A_STATUS_REG_AUTONEGOCIATION_COMPLETE) == 0 )
    {        
        return FALSE;
    }
    
    return TRUE;
}


STATIC BOOL 
PHY_GetID(MAC_PHY_POP pMacPhyOp, BYTE PhyAddr, PUINT32 pID)
{
    UINT32      ID, Value;

    pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_ID1_REG, &Value);
    ID = Value << 16;
    
    Value = 0;
    pMacPhyOp->ReadPhy( pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_ID2_REG, &Value);
    ID |= Value;    

    
    if( pID )
    {
        *pID = ID;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

STATIC BOOL 
PHY_CheckID( MAC_PHY_POP pMacPhyOp, BYTE PhyAddr)
{
#define ID_MASK    0xFFFFF800
    UINT32      ID,ID2;
    int         i =0;
    PHY_GetID( pMacPhyOp,PhyAddr,&ID);
    PHY_GetID( pMacPhyOp,PhyAddr,&ID2);
    
    for( i=0; i<200 && ID==ID2; i++ )
        PHY_GetID( pMacPhyOp, PhyAddr, &ID);
    
/*    if(ID!=ID2)
         DEBUGMSG(1,"PHY ID instable 1: %d, 2: %d.\r\n",ID,ID2);
    else
        DEBUGMSG(1,"PHY ID : %d",ID);
*/
    PHY_GetID( pMacPhyOp, PhyAddr,&ID);
    if( (ID & ID_MASK) != (MII_DM9161A_ID & ID_MASK))
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}


STATIC BOOL 
PHY_SetConfiguration( MAC_PHY_POP pMacPhyOp, BYTE PhyAddr, PHY_PINFO pPhyInfo )
{    
    UINT32  ControlReg;
    UINT32  DavicomSpecifiedConfReg;
    // Check parameters !
    if( pPhyInfo == NULL )
    {
        return FALSE;
    }
    
    pMacPhyOp->ReadPhy( pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_DAVICOM_SPEC_CONF_REG, &DavicomSpecifiedConfReg);
        
    if( (pPhyInfo->bRMII == FALSE)  && (DavicomSpecifiedConfReg & DM9161A_DAVICOM_SPEC_CONF_RMII_ENABLED) )
    {
        DEBUGMSG(1,"PHY_SetConfiguration : MII mode not available\r\n");
        return FALSE;
    }
    if( (pPhyInfo->bRMII == TRUE)  && (!(DavicomSpecifiedConfReg & DM9161A_DAVICOM_SPEC_CONF_RMII_ENABLED)))
    {
        DEBUGMSG(1,"PHY_SetConfiguration : RMII mode not available\r\n");
        return FALSE;
    }

    if( pPhyInfo->bAutoNegociation )
    {
        return PHY_StartAutoNegociation( pMacPhyOp, PhyAddr );
    }

    // Check parameters
    if( (pPhyInfo->Speed != SPEED_100) && 
        (pPhyInfo->Speed != SPEED_10) )
    {
        return FALSE;
    }

    ControlReg = 0;
    if( pPhyInfo->Speed  == SPEED_100 )
    {
        ControlReg |= DM9161A_CONTROL_REG_SPEED_SELECT;
    }
    if( pPhyInfo->bFullDuplex )
    {
        ControlReg |= DM9161A_CONTROL_REG_FULL_DUPLEX;
    }

    pMacPhyOp->WritePhy( pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_CONTROL_REG, ControlReg);    
    
    return FALSE;
}



STATIC BOOL
PHY_GetConfiguration( MAC_PHY_POP pMacPhyOp, BYTE PhyAddr, PHY_PINFO pPhyInfo)
{
    UINT32  CtrlReg;
    UINT32  StatusReg;
    UINT32  LinkPartnerReg;
    UINT32  DavicomSpecifiedConfReg;

    if( pPhyInfo == NULL )
    {
        return FALSE;
    }

    pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_CONTROL_REG, &CtrlReg);
    pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_STATUS_REG, &StatusReg);
    
    DEBUGMSG(0,"CONTROL REG : 0x%x\r\n", CtrlReg);
    DEBUGMSG(0,"STATUS REG : 0x%x\r\n", StatusReg);

    if( CtrlReg & DM9161A_CONTROL_REG_AUTONEGOCIATION_ENABLE ) 
    {    
        pPhyInfo->bAutoNegociation = TRUE;    
        
        /* AutoNegotiation is enabled */
        if( !(StatusReg & DM9161A_STATUS_REG_AUTONEGOCIATION_COMPLETE) ) 
        {
            return FALSE;        /* auto-negotitation in progress */
        }
        
        pMacPhyOp->ReadPhy( pMacPhyOp->pDeviceLocation, PhyAddr, DM9161A_LINK_PARTNER_REG, &LinkPartnerReg);
        if( (LinkPartnerReg & DM9161A_LINK_PARTNER_100FULL) || (LinkPartnerReg & DM9161A_LINK_PARTNER_100HALF) ) 
        {
            pPhyInfo->Speed  = SPEED_100;
        }
        else 
        {
            pPhyInfo->Speed = SPEED_10;
        }
        
        if ((LinkPartnerReg & DM9161A_LINK_PARTNER_100FULL) || (LinkPartnerReg & DM9161A_LINK_PARTNER_10FULL)) 
        {
            pPhyInfo->bFullDuplex = TRUE;
        }        
        else
        {
            pPhyInfo->bFullDuplex = FALSE;
        }        
    } 
    else 
    {
        pPhyInfo->bAutoNegociation = FALSE;
        if (CtrlReg & DM9161A_CONTROL_REG_SPEED_SELECT)
        {
            pPhyInfo->Speed = SPEED_100;
        }
        else
        {    
            pPhyInfo->Speed = SPEED_10;
        }
        if (CtrlReg & DM9161A_CONTROL_REG_FULL_DUPLEX)
        {
            pPhyInfo->bFullDuplex = TRUE;
        }
        else
        {
            pPhyInfo->bFullDuplex = FALSE;
        }
    }

    pMacPhyOp->ReadPhy(pMacPhyOp->pDeviceLocation,PhyAddr,DM9161A_DAVICOM_SPEC_CONF_REG,&DavicomSpecifiedConfReg);
        
    if( DavicomSpecifiedConfReg & DM9161A_DAVICOM_SPEC_CONF_RMII_ENABLED )
    {
        pPhyInfo->bRMII = TRUE;        
    }
    else
    {
        pPhyInfo->bRMII = FALSE;
    }

    return TRUE;
}


STATIC  BYTE PhyName[] = "DM9161A";


PUBLIC PHY_INTERFACE DM9161A_PhyInterface =
{
    PhyName,
    PHY_Reset,
    PHY_SetConfiguration,
    PHY_GetConfiguration,    
    PHY_StartAutoNegociation,
    PHY_WaitForLink,
    PHY_WaitForAutonegociationComplete,
    PHY_GetID,    
    PHY_CheckID,
};



